#include "stdafx.h"
#include "SimpleMixer.h"

class CSndDlg : public CDialogImpl<CSndDlg>,
				public CMessageFilter
{
private:
	HICON m_hOn;
	HICON m_hOff;

	HBITMAP m_bmp;

	CTrackBarCtrl m_volume;
	CButton m_mute;

	CSimpleMixer m_mixer;

	enum { BMP_WIDTH = 11, BMP_HEIGHT = 66 };
	enum { UWM_NOTIFYICON = (WM_APP + 200) };

public:
	enum { IDD = IDD_VOLUME_DIALOG };

	BEGIN_MSG_MAP_EX(CSndDlg)
		MSG_WM_INITDIALOG(OnInitDialog)
		MSG_WM_ACTIVATE(OnActivate)
		MSG_WM_VSCROLL(OnVScroll)
		COMMAND_HANDLER_EX(IDC_MUTE, BN_CLICKED, OnMute)
		COMMAND_ID_HANDLER_EX(ID_EXIT, OnExit)
		MESSAGE_HANDLER_EX(UWM_NOTIFYICON, OnNotifyIcon)
		MESSAGE_HANDLER_EX(MM_MIXM_CONTROL_CHANGE, OnMixerCtrlChange)
	END_MSG_MAP()

	BOOL PreTranslateMessage(MSG *pMsg)
	{
		return IsDialogMessage(pMsg);
	}

	void ShowMute(BOOL bMute)
	{
		m_mute.SetCheck(bMute);

		NOTIFYICONDATA data;
		ZeroMemory(&data, sizeof(data));

		data.cbSize = sizeof(data);
		data.uFlags = NIF_ICON;
		data.hIcon = bMute ? m_hOff : m_hOn;
		data.hWnd = m_hWnd;

		::Shell_NotifyIcon(NIM_MODIFY, &data);
	}

	LRESULT OnInitDialog(HWND, LPARAM)
	{
		m_mixer.Open(0, m_hWnd);

		m_mute = GetDlgItem(IDC_MUTE);
		m_mute.SetCheck(m_mixer.GetMuteCtrlValue());

		m_volume = GetDlgItem(IDC_VOLUME);
		m_volume.SetRangeMin(0);
		m_volume.SetRangeMax(65535);
		m_volume.SetPos(65535-m_mixer.GetVolumeCtrlValue());

		m_hOn = (HICON)::LoadImage
		(
			_Module.GetResourceInstance(),
			MAKEINTRESOURCE(IDI_ON),
			IMAGE_ICON, 0, 0, LR_CREATEDIBSECTION
		);

		m_hOff = (HICON)::LoadImage
		(
			_Module.GetResourceInstance(),
			MAKEINTRESOURCE(IDI_OFF),
			IMAGE_ICON, 0, 0, LR_CREATEDIBSECTION
		);

		NOTIFYICONDATA data;
		ZeroMemory(&data, sizeof(data));

		data.cbSize = sizeof(data);
		data.uFlags = NIF_ICON | NIF_TIP | NIF_MESSAGE;
		data.hIcon = m_mixer.GetMuteCtrlValue() ? m_hOff : m_hOn;
		data.hWnd = m_hWnd;
		data.uCallbackMessage = UWM_NOTIFYICON;
		_tcscpy(data.szTip, _T(""));

		::Shell_NotifyIcon(NIM_ADD, &data);

		HDC hdc = GetDC();
		HDC hmemdc = ::CreateCompatibleDC(hdc);
		m_bmp = ::CreateCompatibleBitmap(hdc, BMP_WIDTH, BMP_HEIGHT);
		HPEN hLightPen = ::CreatePen(PS_SOLID, 0, GetSysColor(COLOR_3DHILIGHT));
		HPEN hDarkPen = ::CreatePen(PS_SOLID, 0, GetSysColor(COLOR_3DSHADOW));

		::SaveDC(hmemdc);		
		
		::SelectObject(hmemdc, m_bmp);
		::SelectObject(hmemdc, GetSysColorBrush(COLOR_3DFACE));

		//   ,     .
		::SetBkColor(hmemdc, GetSysColor(COLOR_3DFACE));
		RECT rect = { 0, 0, BMP_WIDTH, BMP_HEIGHT };
		::ExtTextOut(hmemdc, 0, 0, ETO_OPAQUE, &rect, NULL, 0, NULL);

		::SelectObject(hmemdc, hLightPen);
		::MoveToEx(hmemdc, BMP_WIDTH-1, 0, NULL);
		::LineTo(hmemdc, 0, 0);
		::LineTo(hmemdc, BMP_WIDTH-1, BMP_HEIGHT-1);

		::SelectObject(hmemdc, hDarkPen);
		::LineTo(hmemdc, BMP_WIDTH-1, 0);

		::RestoreDC(hmemdc, -1);

		::DeleteObject(hLightPen);
		::DeleteObject(hDarkPen);

		CStatic st = GetDlgItem(IDC_BITMAP);
		st.SetBitmap(m_bmp);

		//      .
		//    ,   IsDialogMessage   .
		_Module.GetMessageLoop()->AddMessageFilter(this);

		return 0;
	}

	LRESULT OnActivate(UINT, BOOL bActivate, HWND)
	{
		if(bActivate == FALSE)
		{
			ShowWindow(SW_HIDE);
		}

		return 0;
	}

	LRESULT OnVScroll(int, short, HWND)
	{
		m_mixer.SetVolumeCtrlValue(65535-m_volume.GetPos());
		return 0;
	}

	LRESULT OnMute(UINT, int, HWND)
	{
		BOOL bMute = m_mute.GetCheck();
		m_mixer.SetMuteCtrlValue(bMute);
		ShowMute(bMute);
		return 0;
	}

	LRESULT OnExit(UINT, int, HWND)
	{
		_Module.GetMessageLoop()->RemoveMessageFilter(this);

		NOTIFYICONDATA data;
		ZeroMemory(&data, sizeof(data));

		data.cbSize = sizeof(data);
		data.hWnd = m_hWnd;

		::Shell_NotifyIcon(NIM_DELETE, &data);

		DestroyWindow();
		PostQuitMessage(0);
		return 0;
	}

	LRESULT OnNotifyIcon(UINT, WPARAM wParam, LPARAM lParam)
	{
		if(lParam == WM_RBUTTONUP)
		{
			HMENU hMenu = CreatePopupMenu();
			AppendMenu(hMenu, MF_STRING, ID_EXIT, _T("Exit"));

			::SetForegroundWindow(m_hWnd);
			POINT pt;
			::GetCursorPos(&pt);
			::TrackPopupMenu(hMenu, 0, pt.x, pt.y, 0, m_hWnd, NULL);
			::DestroyMenu(hMenu);

			PostMessage(WM_NULL, 0, 0);
		}
		else if(lParam == WM_LBUTTONDOWN)
		{
			Sleep(1000);

			POINT pt;
			::GetCursorPos(&pt);

			RECT rt;
			GetWindowRect(&rt);
			::OffsetRect(&rt, pt.x-rt.left, pt.y-rt.top);

			//      ,  .
			int scrWidth = ::GetSystemMetrics(SM_CXSCREEN);
			int scrHeight = ::GetSystemMetrics(SM_CYSCREEN);

			if(rt.right > scrWidth) ::OffsetRect(&rt, rt.left - rt.right, 0);
			if(rt.bottom > scrHeight) ::OffsetRect(&rt, 0, rt.top - rt.bottom);

			SetWindowPos(HWND_TOPMOST, &rt, SWP_NOSIZE);
			ShowWindow(SW_SHOW);
			BringWindowToTop();
			SetFocus();
			::SetForegroundWindow(m_hWnd);
		}

		return 0;
	}

	LRESULT OnMixerCtrlChange(UINT, WPARAM wParam, LPARAM lParam)
	{
		DWORD dwCtrlID = (DWORD)lParam;

		if(m_mixer.m_dwMuteCtrlID == dwCtrlID)
		{
			//    .
			ShowMute(m_mixer.GetMuteCtrlValue());
		}
		else if(m_mixer.m_dwVolumeCtrlID == dwCtrlID)
		{
			//    .
			m_volume.SetPos(65535-m_mixer.GetVolumeCtrlValue());
		}

		return 0;
	}
};